@@ -1,5 +1,5 @@
 
-::Version::setSubversionId {$Id: config.tcl 9083 2007-11-04 15:17:19Z baaazen $}
+::Version::setSubversionId {$Id: config.tcl 10049 2008-05-28 22:28:36Z baaazen $}
 
 
 
@@ -58,7 +58,7 @@ namespace eval ::config {
 
 		#Dir for received files
 		if { [OnDarwin] } {
-		        ::config::setKey receiveddir "[file join $::env(HOME) Desktop/amsn\ received\ files]"
+		        ::config::setKey receiveddir "[file join $::env(HOME) Desktop]"
 		} elseif { [OnUnix] } {
 			::config::setKey receiveddir "[file join $::env(HOME) amsn_received]"
 		} elseif { [OnWin] } {
@@ -290,6 +290,10 @@ namespace eval ::config {
 
 		::config::setKey escape_close_cw 1				;# Whether the escape key closes the chat windows or not
 
+		::config::setKey dynamic_rate 0 				;# Use a dynamic framerate for webcam depending on the timestamps in the ML20 header
+
+                ::config::setKey no_oim_confirmation 0                          ;# Ask or not confirmation to send/read oim messages
+
 
 		#Advanced options, not in preferences window
 		# Create the entry in the list and then, set
@@ -359,6 +363,7 @@ namespace eval ::config {
 			[list local storename bool storenickname] \
 			[list local globaloverride bool globaloverride ] \
 			[list local escape_close_cw bool escapeclosescw ] \
+			[list local no_oim_confirmation bool nooimconfirmation ] \
 			[list global disableprofiles bool disableprofiles] \
 		]
 
@@ -529,9 +534,9 @@ proc save_config {} {
 
 	if { [catch {
 		if { [OnUnix] } {
-			set file_id [open "[file join ${HOME} config.xml]" w 00600]
+			set file_id [open "[file join ${HOME} config.xml.temp]" w 00600]
 		} else {
-			set file_id [open "[file join ${HOME} config.xml]" w]
+			set file_id [open "[file join ${HOME} config.xml.temp]" w]
 		}
 	} res]} {
 		return 0
@@ -590,6 +595,9 @@ proc save_config {} {
 	puts $file_id "</config>"
 
 	close $file_id
+	
+	#writing was successful, so move config.xml.temp to config.xml
+	file rename -force [file join ${HOME} config.xml.temp] [file join ${HOME} config.xml]
 
 	::config::setKey login $loginback
 	set password $passback
@@ -1161,6 +1169,7 @@ proc SwitchToDefaultProfile { } {
 	::config::setKey log_event_state 0
 	::config::setKey log_event_nick 0
 	::config::setKey log_event_psm 0
+	::config::setKey displaypic "amsn.png"
 }
 
 #///////////////////////////////////////////////////////////////////////////////
@@ -1247,6 +1256,7 @@ proc CreateProfile { email } {
 
 	set oldpassword $password
 	set oldsavepwd [::config::getKey save_password]
+	set oldautoconnect [::config::getKey autoconnect 0]
 
 	# set all config keys to default
 	::config::configDefaults
@@ -1255,10 +1265,12 @@ proc CreateProfile { email } {
 	::config::setKey login $email
 	set password $oldpassword
 	::config::setKey save_password $oldsavepwd
+	::config::setKey autoconnect $oldautoconnect
 	save_config
 
 	unset oldpassword
 	unset oldsavepwd
+	unset oldautoconnect
 
 	# Add to login list
 	LoginList add 0 $email 0
@@ -1357,8 +1369,8 @@ proc CheckLock { email } {
 	set Port [LoginList getlock 0 $email]
 	status_log "CheckLock: LoginList getlock called. Lock=$Port\n" blue
 	if { $Port != 0 } {
-		if { [catch {socket -server phony $Port} newlockSock] != 0  } {
-			status_log "CheckLock Port is already in use: $newlockSock\n" red
+		if { [OnWinVista] || [catch {socket -server phony $Port} newlockSock] != 0 } {
+			#status_log "CheckLock Port is already in use: $newlockSock\n" red
 			# port is taken, let's make sure it's a profile lock
 			foreach {local_host} { localhost "[info hostname]" 127.0.0.1 } {
 				if {[catch {socket $local_host $Port} clientSock] == 0 } {
@@ -1430,7 +1442,7 @@ proc LockProfile { email } {
 	status_log "LockProfile: Locking $email\n" blue
 	global lockSock
 	set tries 0
-	while { $tries < 5 } {
+	while { $tries < 6 } {
 		set Port [GetRandomProfilePort]
 		status_log "LockProfile: Got random port $Port\n" blue
 		if { [::config::getKey enableremote] == 1 } {
@@ -1443,8 +1455,8 @@ proc LockProfile { email } {
 			set lockSock $newlockSock
 			break
 		} elseif {$tries >= 5} {
-			::amsn::errorMsg "Unable to get a socket from locahost.\n Check your /etc/hosts file, please."
-			exit 1
+			::amsn::errorMsg "Unable to get a socket from localhost.\n Check your /etc/hosts file, please. Be sure to have net loopback up."
+			break
 		} else {
 			catch {status_log "unable to create socket on port $Port - $newlockSock"}
 			incr tries
@@ -1487,6 +1499,20 @@ proc lockSvrHdl { addr sock } {
 	}
 }
 
+proc WinDetectBoot { } {
+	if { [OnWin] } {
+		if {[catch {package require registry}] } {
+			return 0
+		}
+		set path "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"
+		if {[catch { registry get $path amsn} ] } {
+			return 0
+		} else {
+			return 1
+		}
+	}
+	return 0
+}
 #///////////////////////////////////////////////////////////////////////
 # WinRegKey ( [addrem] )
 # Adds or removes the registry key to start amsn on startup (windows only)
@@ -1496,31 +1522,21 @@ proc lockSvrHdl { addr sock } {
 #         file after completion, but the registry key may have still been created/removed
 proc WinRegKey { addrem } {
 	if { [OnWin] } {
-	    set filename [file join $::env(TEMP) amsn_addrem.reg]
-		if { [catch { set file_id [open "$filename" w]} res]} {
-			msg_box "Failed to create temporary file with error:\n$res"
+		if {[catch {package require registry}] } {
 			return 0
 		}
-		puts $file_id "REGEDIT4\n"
-		puts $file_id "\[HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run\]"
-		if { $addrem == "add" } {
-		    set amsn_path [file nativename [file normalize [file join [file dirname [info nameofexecutable]] .. amsn.exe]]]
-		    puts $file_id "\"amsn\"=\"\\\"[string map {"\\" "\\\\"} $amsn_path]\\\"\"\n"
+		set path "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"
+		set amsn_path [file nativename [file normalize [file join [file dirname [info nameofexecutable]] .. amsn.exe]]]
+		if {![file exists $amsn_path] } {
+			set amsn_path "[file nativename [file normalize [info nameofexecutable]]] [file nativename [file normalize [info script]]]"
+		}
+	
+		if { $addrem } {
+			catch {registry set $path amsn $amsn_path}
 		} else {
-		    puts $file_id "\"amsn\"=-"
+			catch {registry delete $path amsn}
 		}
 		
-		close $file_id
-
-		if { [catch { exec regedit /s "$filename"} res]} {
-			msg_box "Failed to create/remove the registry key with error:\n$res"
-			return 0
-		}
-		
-		if { [catch { file delete "$filename"} res]} {
-			msg_box "Failed to delete temporary file with error:\n$res"
-			return 0
-		}
 		return 1
 	}
 	return 0
