@@ -1,5 +1,5 @@
 
-::Version::setSubversionId {$Id: msnp2p.tcl 9143 2007-12-22 19:26:43Z kakaroto $}
+::Version::setSubversionId {$Id: msnp2p.tcl 10013 2008-05-28 13:11:30Z baaazen $}
 
 ::snit::type P2PMessage {
 
@@ -186,8 +186,11 @@ namespace eval ::MSNP2P {
 			create_dir [file join $HOME displaypic cache]
 			::MSNP2P::RequestObject $chatid $user $msnobj
 		} else {
-			::skin::getDisplayPicture $user 1
+			::skin::getDisplayPicture $user 1	
 		}
+		# launch an event for plugins
+		set evPar(user) $user
+		::plugins::PostEvent ChangeDP evPar
 	}
 
 	proc loadUserSmiley { chatid user msnobj } {
@@ -277,20 +280,20 @@ namespace eval ::MSNP2P {
 
 		switch $action {
 			get {
-				set ret [getObjOption $sid MsgId] 
+				set ret [getObjOption sl_$sid MsgId] 
 				#status_log "getting $sid : [getObjOption $sid MsgId] - $ret" green
 
 				if { $ret != "" } {
 					# Session found, return values
-					lappend ret [getObjOption $sid TotalSize] 
-					lappend ret [getObjOption $sid Offset] 
-					lappend ret [getObjOption $sid Destination] 
-					lappend ret [getObjOption $sid AfterAck]
-					lappend ret [getObjOption $sid CallId] 
-					lappend ret [getObjOption $sid Fd]
-					lappend ret [getObjOption $sid Type] 
-					lappend ret [getObjOption $sid Filename]
-					lappend ret [getObjOption $sid branchid]
+					lappend ret [getObjOption sl_$sid TotalSize] 
+					lappend ret [getObjOption sl_$sid Offset] 
+					lappend ret [getObjOption sl_$sid Destination] 
+					lappend ret [getObjOption sl_$sid AfterAck]
+					lappend ret [getObjOption sl_$sid CallId] 
+					lappend ret [getObjOption sl_$sid Fd]
+					lappend ret [getObjOption sl_$sid Type] 
+					lappend ret [getObjOption sl_$sid Filename]
+					lappend ret [getObjOption sl_$sid branchid]
 
 					#status_log "returning $ret" green
 					return $ret
@@ -305,49 +308,64 @@ namespace eval ::MSNP2P {
 				#status_log "setting $sid with $varlist" green
 				# This overwrites previous vars if they are set to something else than -1
 				if { [lindex $varlist 0] != -1 } {
-					setObjOption $sid MsgId [lindex $varlist 0]
-					setObjOption [lindex $varlist 0] sid $sid
+					set MsgsIds [getObjOption sl_${sid} MsgsIds]
+					lappend MsgsIds [lindex $varlist 0]
+					setObjOption sl_$sid MsgId [lindex $varlist 0]
+					setObjOption sl_$sid MsgsIds $MsgsIds
+					setObjOption slm_[lindex $varlist 0] sid $sid
 				}
 				if { [lindex $varlist 1] != -1 } {
-					setObjOption $sid TotalSize [lindex $varlist 1]
+					setObjOption sl_$sid TotalSize [lindex $varlist 1]
 				}
 				if { [lindex $varlist 2] != -1 } {
-					setObjOption $sid Offset [lindex $varlist 2]
+					setObjOption sl_$sid Offset [lindex $varlist 2]
 				}
 				if { [lindex $varlist 3] != -1 } {
-					setObjOption $sid Destination [lindex $varlist 3]
+					setObjOption sl_$sid Destination [lindex $varlist 3]
 				}
 				if { [lindex $varlist 4] != -1 } {
-					setObjOption $sid AfterAck [lindex $varlist 4]
+					setObjOption sl_$sid AfterAck [lindex $varlist 4]
 				}
 				if { [lindex $varlist 5] != -1 } {
-					setObjOption $sid CallId [lindex $varlist 5]
-					setObjOption [lindex $varlist 5] sid $sid
+					set CallsIds [getObjOption sl_${sid} CallsIds]
+					lappend CallsIds [lindex $varlist 0]
+					setObjOption sl_$sid CallId [lindex $varlist 5]
+					setObjOption sl_$sid CallsIds $CallsIds
+					setObjOption slc_[lindex $varlist 5] sid $sid
 				}
 				if { [lindex $varlist 6] != -1 } {
-					setObjOption $sid Fd [lindex $varlist 6]
+					setObjOption sl_$sid Fd [lindex $varlist 6]
 				}
 				if { [lindex $varlist 7] != -1 } {
-					setObjOption $sid Type [lindex $varlist 7]
+					setObjOption sl_$sid Type [lindex $varlist 7]
 				}
 				if { [lindex $varlist 8] != -1 } {
-				        setObjOption $sid Filename [lindex $varlist 8]
+				        setObjOption sl_$sid Filename [lindex $varlist 8]
 				}
 				if { [lindex $varlist 9] != -1 } {
-				        setObjOption $sid branchid [lindex $varlist 9]
+				        setObjOption sl_$sid branchid [lindex $varlist 9]
 				}
 			}
 
 			unset {
 				#status_log "unsetting..." green
+				set msgsids [getObjOption sl_$sid MsgsIds] 
+				set callsids [getObjOption sl_$sid CallsIds] 
+				clearObjOption sl_$sid
+				foreach msgid $msgsids {
+					clearObjOption slm_$msgid
+				}
+				foreach callid $callsids {
+					clearObjOption slc_$callid
+				}
 				return
 			}
-			findcallid -
+			findcallid {
+				return [getObjOption slc_$sid sid]
+			}
 			findid {
 				#status_log "Finding $action of $sid, found : [getObjOption $sid sid]" green
-				if { [getObjOption $sid sid] != "" } {
-					return [getObjOption $sid sid]
-				}
+				return [getObjOption slm_$sid sid]
 			}
 		}
 	}
@@ -445,35 +463,47 @@ namespace eval ::MSNP2P {
 			set sid [SessionList findid $cAckId]
 			status_log "GOT SID : $sid for Ackid : $cAckId\n"
 			if { $sid != -1 } {
-			 	#status_log "MSNP2P | $sid -> Got MSNP2P ACK " red
+				if { $cFlags == 2 } {
+					#status_log "MSNP2P | $sid -> Got MSNP2P ACK " red
 
-			 	# We found a session id that is waiting for an ACK
-			 	set step [lindex [SessionList get $sid] 4]
+					# We found a session id that is waiting for an ACK
+					set step [lindex [SessionList get $sid] 4]
 
-				# Just these 2 for now, will probably need more with file transfers
-				switch $step {
-					DATAPREP {
-						# Set the right variables, prepare to send data after next ack
-						SessionList set $sid [list -1 4 0 -1 "SENDDATA" -1 -1 -1 -1 -1]
-
-						# We need to send a data preparation message
-						SendPacket [::MSN::SBFor $chatid] [MakePacket $sid [binary format i 0]]
-						status_log "MSNP2P | $sid -> Sent DATA Preparation\n" red
-					}
-					SENDDATA {
-						#status_log "MSNP2P | $sid -> Sending DATA now\n" red
-						set file [lindex [SessionList get $sid] 8]
-						if { $file != "" } {
-							SendData $sid $chatid "[lindex [SessionList get $sid] 8]"
-						} else {
-							SendData $sid $chatid "[::skin::GetSkinFile displaypic [::config::getKey displaypic]]"
+					# Just these 2 for now, will probably need more with file transfers
+					switch $step {
+						DATAPREP {
+							# Set the right variables, prepare to send data after next ack
+							SessionList set $sid [list -1 4 0 -1 "SENDDATA" -1 -1 -1 -1 -1]
+							
+							# We need to send a data preparation message
+							SendPacket [::MSN::SBFor $chatid] [MakePacket $sid [binary format i 0]]
+							status_log "MSNP2P | $sid -> Sent DATA Preparation\n" red
+						}
+						SENDDATA {
+							#status_log "MSNP2P | $sid -> Sending DATA now\n" red
+							set file [lindex [SessionList get $sid] 8]
+							if { $file != "" } {
+								SendData $sid $chatid "[lindex [SessionList get $sid] 8]"
+							} else {
+								SendData $sid $chatid "[::skin::GetSkinFile displaypic [::config::getKey displaypic]]"
+							}
+						}
+						DATASENT {
+							SessionList set $sid [list -1 -1 0 -1 "BYE" -1 -1 -1 -1 -1]
+							#status_log "MSNP2P | $sid -> Got ACK for sending data, now sending BYE\n" red
+							set branchid "[format %X [myRand 4369 65450]][format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]][format %X [myRand 4369 65450]][format %X [myRand 4369 65450]]"
+							SendPacket [::MSN::SBFor $chatid] [MakePacket $sid [MakeMSNSLP "BYE" [lindex [SessionList get $sid] 3] [::config::getKey login] "$branchid" "0" [lindex [SessionList get $sid] 5] 0 0] 1]
+						
+						}
+						BYE {
+							SessionList unset $sid
 						}
 					}
-					DATASENT {
-						SessionList set $sid [list -1 -1 0 -1 0 -1 -1 -1 -1 -1]
-						#status_log "MSNP2P | $sid -> Got ACK for sending data, now sending BYE\n" red
-						set branchid "[format %X [myRand 4369 65450]][format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]]-[format %X [myRand 4369 65450]][format %X [myRand 4369 65450]][format %X [myRand 4369 65450]]"
-						SendPacket [::MSN::SBFor $chatid] [MakePacket $sid [MakeMSNSLP "BYE" [lindex [SessionList get $sid] 3] [::config::getKey login] "$branchid" "0" [lindex [SessionList get $sid] 5] 0 0] 1]
+				} elseif { $cFlags == 1 } {
+					set blobid [getObjOption $sid data_blob_id]
+					if {$cAckId == $blobid } {
+						status_log "Received a NAK on our data being sent. Need to reposition ourselves... to $cAckSize"
+						setObjOption $sid nak_pos $cAckSize
 					}
 				}
 			}
@@ -856,12 +886,25 @@ namespace eval ::MSNP2P {
 
 				# Delete SessionID Data
 				SessionList unset $sid
+				clearObjOption $sid
 
 			} else {
 				status_log "MSNP2P | $sid -> Got a BYE for unexisting SessionID\n" red
 			}
 			return
 		}
+		if { [string first "ACK MSNMSGR:" $data] != -1 } {
+			set idx [expr {[string first "SessionID:" $data] + 11}]
+			set idx2 [expr {[string first "\r\n" $data $idx] -1}]
+			set sid [string range $data $idx $idx2]
+
+			# We get an ACK message for receiving the user's ip:port for a file transfer..
+			# if we don't ack this message, the FT gets canceled...
+			# TODO : actually parse the message and use it.. RE necessary.
+			SendPacket [::MSN::SBFor $chatid] [MakeACK $sid 0 $cTotalDataSize $cId $cAckId]
+			return
+		}
+
 		# Check if we got DECLINE message
 		if { [string first "603 Decline" $data] != -1 } {
 			# Lets get the call ID and find our SessionID
@@ -881,7 +924,8 @@ namespace eval ::MSNP2P {
 				    set idx [expr {[string first "Content-Type: " $data] + 14}]
 				    set idx2 [expr {[string first "\r\n" $data $idx] - 1}]
 				    set content_type [string range $data $idx $idx2]
-				    if { $content_type != "null" } {
+				    if { $content_type != "null"  &&
+					 $content_type != "application/x-msnmsgr-session-failure-respbody" } {
 					::CAMGUI::InvitationDeclined $chatid $sid
 				    } else {
 					::MSNCAM::SendSyn $sid $chatid
@@ -938,6 +982,7 @@ namespace eval ::MSNP2P {
 
 					if { [lindex [SessionList get $cSid] 7] == "bicon" } {
 						SendPacket [::MSN::SBFor $chatid] [MakePacket $sid [MakeMSNSLP "BYE" $user_login [::config::getKey login] "19A50529-4196-4DE9-A561-D68B0BF1E83F" 0 [lindex $session_data 5] 0 0] 1]
+						::MSNP2P::SessionList set $sid [list -1 -1 -1 -1 "BYE" -1 -1 -1 -1 -1]
 
 						#set filename2 [::MSNP2P::GetFilenameFromMSNOBJ [::abook::getVolatileData $user_login msnobj]]
 						set filename2 [::abook::getContactData $user_login displaypicfile ""]
@@ -956,6 +1001,8 @@ namespace eval ::MSNP2P {
 #							puts $fd "[clock format [clock seconds] -format %x]\n$user_login"
 							puts $fd "[clock seconds]\n$user_login"
 							close $fd
+							# new DP, maybe we need to refresh the CL
+							::Event::fireEvent contactDPChange protocol $user_login
 
 						} else {
 							#set file [png_to_gif [file join $HOME smileys cache ${filename}.png]]
@@ -1030,11 +1077,12 @@ namespace eval ::MSNP2P {
 				} elseif {$type == "wink" }  {
 					set fd [open "[file join $HOME winks cache ${filename}.cab]" w]
 				}
-
-				fconfigure $fd -translation {binary binary}
-				SendPacket [::MSN::SBFor $chatid] [MakeACK $sid $cSid $cTotalDataSize $cId $cAckId]
-				#status_log "MSNP2P | $sid $user_login -> Sent an ACK for DATA PREP Message\n" red
-				SessionList set $sid [list -1 -1 -1 -1 -1 -1 $fd -1 -1 -1]
+				if {$fd != "" } {
+					fconfigure $fd -translation {binary binary}
+					SendPacket [::MSN::SBFor $chatid] [MakeACK $sid $cSid $cTotalDataSize $cId $cAckId]
+					#status_log "MSNP2P | $sid $user_login -> Sent an ACK for DATA PREP Message\n" red
+					SessionList set $sid [list -1 -1 -1 -1 -1 -1 $fd -1 -1 -1]
+				}
 			} elseif { $type == "webcam" } {
 				# WEBCAM TO COMPLETE
 
@@ -1167,14 +1215,26 @@ namespace eval ::MSNP2P {
 	# nullsid 	: 0 to add sid to header, 1 to put 0 instead of sid in header (usefull for negot + bye)
 	# Returns the MSNP2P packet (half text half binary)
 	proc MakePacket { sid slpdata {nullsid "0"} {MsgId "0"} {TotalSize "0"} {Offset "0"} {Destination "0"} {AfterAck "0"} {flags "0"}} {
+		set custom_msgid 1
+		set custom_totalsize 1
+		set custom_offset 1
 
 		# Let's get our session id variables and put them in a list
 		# If sessionid is 0, means we want to initiate a new session
 		if { $sid != 0 } {
 			set SessionInfo [SessionList get $sid]
-			set MsgId [lindex $SessionInfo 0]
-			set TotalSize [lindex $SessionInfo 1]
-			set Offset [lindex $SessionInfo 2]
+			if {$MsgId == "0" } {
+				set MsgId [lindex $SessionInfo 0]
+				set custom_msgid 0
+			}
+			if {$TotalSize == "0" } {
+				set TotalSize [lindex $SessionInfo 1]
+				set custom_totalsize 0
+			}
+			if {$Offset == "0" } {
+				set Offset [lindex $SessionInfo 2]
+				set custom_offset 0
+			}
 			set Destination [lindex $SessionInfo 3]
 		}
 
@@ -1251,6 +1311,18 @@ namespace eval ::MSNP2P {
 		unset bfooter
 		unset slpdata
 
+		# Reset variables if we use the custom ones to avoid overriding the ones in SessionList
+		# and corrupt our stack
+		if {$custom_msgid } {
+			set MsgId -1
+		} 
+		if {$custom_totalsize } {
+			set TotalSize -1
+		}
+		if {$custom_offset } {
+			set Offset -1
+		}
+
 		# Save new Session Variables into SessionList
 		SessionList set $sid [list $MsgId $TotalSize $Offset -1 -1 -1 -1 -1 -1 -1]
 
@@ -1426,7 +1498,7 @@ namespace eval ::MSNP2P {
 		unset chunk
 
 		#status_log "[SessionList get $sid]\n"
-		if { [lindex [SessionList get $sid] 1] == 0 } {
+		if { [eof $fd] || [lindex [SessionList get $sid] 1] == 0 } {
 			# All file has been sent
 			close $fd
 			unset fd
@@ -1513,10 +1585,24 @@ namespace eval ::MSNP2P {
 			return
 		}
 
+		set MsgId [getObjOption $sid data_blob_id 0]
+
+		# We can receive a NAK to our data being sent.
+		# When a packet is lost for some reason, we get a nak with the position
+		# of the file that WLM wants to receive. If we got that, we simply reposition 
+		# ourselves in the file and set the new offset.
+		if {[getObjOption $sid nak_pos ""] != "" } {
+			set new_pos [getObjOption $sid nak_pos]
+			set offset $new_pos
+			SessionList set $sid [list -1 -1 $offset -1 -1 -1 -1 -1 -1 -1]
+			seek $fd $new_pos
+			setObjOption $sid nak_pos ""
+		}
+
 		set data [read $fd 1202]
 		#		status_log "Reading 1202 bytes of data : got [string length $data]"
 		if { [string length $data] >= 1202 } {
-			set msg [MakePacket $sid $data 0 0 0 0 0 0 16777264]
+			set msg [MakePacket $sid $data 0 $MsgId 0 0 0 0 16777264]
 			set msg_len [string length $msg]
 			puts -nonewline $sock "MSG [incr ::MSN::trid] D $msg_len\r\n$msg"
 			set offset [expr {$offset + 1202}]
@@ -1527,7 +1613,7 @@ namespace eval ::MSNP2P {
 			
 		} else {
 
-			set msg [MakePacket $sid $data 0 0 0 0 0 0 16777264]
+			set msg [MakePacket $sid $data 0 $MsgId 0 0 0 0 16777264]
 			set msg_len [string length $msg]
 			puts -nonewline $sock "MSG [incr ::MSN::trid] D $msg_len\r\n$msg"
 			set offset [expr {$offset + 1202}]
@@ -1540,6 +1626,10 @@ namespace eval ::MSNP2P {
 
 			::amsn::FTProgress fs $sid ""
 		}
+		if {$MsgId == 0 } {
+			set MsgId [lindex [SessionList get $sid] 0]			
+			setObjOption $sid data_blob_id $MsgId
+		}
 	}
 	
 	proc SetSendDataFileEvent { sock sbn sid fd match} { 
